@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -111,7 +111,6 @@ copy_environ (char **env, ...)
   return ret.ptr;
 
  error:
-  string_vector_iter (&ret, (void *) free);
-  free (ret.ptr);
+  string_vector_empty (&ret);
   return NULL;
 }
