@@ -97,7 +97,7 @@ sh_thread_model (void)
   int r;
 
   /* For historical compatibility: the lack of a script is assumed to
-   * be parallel, but an existing script with missing or unparseable
+   * be parallel, but an existing script with missing or unparsable
    * thread_model remains at the older (but safe)
    * serialize_all_requests.
    */
@@ -1041,17 +1041,18 @@ parse_extents (const char *script,
       len--;
     }
 
-    if ((p = strtok_r (line, delim, &sp)) == NULL) {
-    parse_error:
-      nbdkit_error ("%s: extents: cannot parse %s", script, line);
-      goto out;
-    }
+    if ((p = strtok_r (line, delim, &sp)) == NULL)
+      continue;
+    if (*p == '#')
+      continue;
     offset = nbdkit_parse_size (p);
     if (offset == -1)
       goto out;
 
-    if ((p = strtok_r (NULL, delim, &sp)) == NULL)
-      goto parse_error;
+    if ((p = strtok_r (NULL, delim, &sp)) == NULL) {
+      nbdkit_error ("%s: extents: cannot parse %s", script, line);
+      goto out;
+    }
     length = nbdkit_parse_size (p);
     if (length == -1)
       goto out;
