@@ -105,6 +105,25 @@ ocaml_nbdkit_parse_bool (value strv)
   CAMLreturn (rv);
 }
 
+NBDKIT_DLL_PUBLIC value
+ocaml_nbdkit_parse_delay (value whatv, value strv)
+{
+  CAMLparam2 (whatv, strv);
+  CAMLlocal1 (rv);
+  unsigned sec, nsec;
+  int r;
+
+  r = nbdkit_parse_delay (String_val (whatv), String_val (strv),
+                          &sec, &nsec);
+  if (r == -1)
+    caml_invalid_argument ("nbdkit_parse_delay");
+  rv = caml_alloc (2, 0);
+  Store_field (rv, 0, Val_int (sec));
+  Store_field (rv, 1, Val_int (nsec));
+
+  CAMLreturn (rv);
+}
+
 NBDKIT_DLL_PUBLIC value
 ocaml_nbdkit_read_password (value strv)
 {
@@ -328,3 +347,27 @@ ocaml_nbdkit_peer_security_context (value unitv)
   free (label);
   CAMLreturn (rv);
 }
+
+NBDKIT_DLL_PUBLIC value
+ocaml_nbdkit_peer_tls_dn (value unitv)
+{
+  CAMLparam1 (unitv);
+  CAMLlocal1 (rv);
+  char *label = nbdkit_peer_tls_dn ();
+  if (label == NULL) caml_failwith ("nbdkit_peer_tls_dn");
+  rv = caml_copy_string (label);
+  free (label);
+  CAMLreturn (rv);
+}
+
+NBDKIT_DLL_PUBLIC value
+ocaml_nbdkit_peer_tls_issuer_dn (value unitv)
+{
+  CAMLparam1 (unitv);
+  CAMLlocal1 (rv);
+  char *label = nbdkit_peer_tls_issuer_dn ();
+  if (label == NULL) caml_failwith ("nbdkit_peer_tls_issuer_dn");
+  rv = caml_copy_string (label);
+  free (label);
+  CAMLreturn (rv);
+}
