@@ -208,7 +208,7 @@ extern struct backend *top;
   } while (0)
 
 extern void debug_in_server (const char *msg, ...)
-  ATTRIBUTE_FORMAT_PRINTF (1, 2);
+  NBDKIT_ATTRIBUTE_FORMAT_PRINTF (1, 2);
 
 #define nbdkit_debug _Pragma("GCC error \"Use debug() instead of nbdkit_debug() inside the server\"")
 
@@ -240,9 +240,9 @@ extern unsigned int get_socket_activation (void);
 /* timeout.c */
 struct connection;
 extern int start_timeout (struct connection *conn)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern void cancel_timeout (struct connection *conn)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 /* usergroup.c */
 extern void change_user (void);
@@ -258,10 +258,10 @@ enum {
 };
 
 typedef int (*connection_recv_function) (void *buf, size_t len)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 typedef int (*connection_send_function) (const void *buf, size_t len,
                                          int flags)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 typedef void (*connection_close_function) (int how);
 
 /* struct context stores data per connection and backend.  Primarily
@@ -363,7 +363,7 @@ extern bool connection_set_status (conn_status value);
 extern int protocol_handshake (void);
 extern int protocol_common_open (uint64_t *exportsize, uint16_t *flags,
                                  const char *exportname)
-  __attribute__ ((__nonnull__ (1, 2, 3)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 3));
 
 /* protocol-handshake-oldstyle.c */
 extern int protocol_handshake_oldstyle (void);
@@ -400,9 +400,9 @@ extern void log_verror (const char *fs, va_list args);
 /* log-*.c */
 extern void log_fp_verror (FILE *fp,
                            int orig_errno, const char *fs, va_list args)
-  ATTRIBUTE_FORMAT_PRINTF (3, 0);
+  NBDKIT_ATTRIBUTE_FORMAT_PRINTF (3, 0);
 extern void log_syslog_verror (int orig_errno, const char *fs, va_list args)
-  ATTRIBUTE_FORMAT_PRINTF (2, 0);
+  NBDKIT_ATTRIBUTE_FORMAT_PRINTF (2, 0);
 
 /* vfprintf.c */
 #if !HAVE_VFPRINTF_PERCENT_M
@@ -500,18 +500,18 @@ struct backend {
 
 extern void backend_init (struct backend *b, struct backend *next, size_t index,
                           const char *filename, void *dl, const char *type)
-  __attribute__ ((__nonnull__ (1, 4, 5, 6)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 4, 5, 6));
 extern void backend_load (struct backend *b, const char *name,
                           void (*load) (void))
-  __attribute__ ((__nonnull__ (1 /* not 2 */)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1 /* not 2 */));
 extern void backend_unload (struct backend *b, void (*unload) (void))
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 extern int backend_list_exports (struct backend *b, int readonly,
                                  struct nbdkit_exports *exports)
-  __attribute__ ((__nonnull__ (1, 3)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3));
 extern const char *backend_default_export (struct backend *b, int readonly)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 /* exportname is only valid for this call and almost certainly will be
  * freed on return of this function, so backends must save the
  * exportname if they need to refer to it later.
@@ -519,87 +519,87 @@ extern const char *backend_default_export (struct backend *b, int readonly)
 extern struct context *backend_open (struct backend *b,
                                      int readonly, const char *exportname,
                                      int shared)
-  __attribute__ ((__nonnull__ (1, 3)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3));
 extern int backend_prepare (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_finalize (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern void backend_close (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern bool backend_valid_range (struct context *c,
                                  uint64_t offset, uint32_t count)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 extern const char *backend_export_description (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int64_t backend_get_size (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_block_size (struct context *c,
                                uint32_t *minimum, uint32_t *preferred,
                                uint32_t *maximum)
-  __attribute__ ((__nonnull__ (1, 2, 3, 4)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 3, 4));
 extern int backend_can_write (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_flush (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_is_rotational (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_trim (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_zero (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_fast_zero (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_extents (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_fua (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_multi_conn (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern int backend_can_cache (struct context *c)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 extern int backend_pread (struct context *c,
                           void *buf, uint32_t count, uint64_t offset,
                           uint32_t flags, int *err)
-  __attribute__ ((__nonnull__ (1, 2, 6)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 6));
 extern int backend_pwrite (struct context *c,
                            const void *buf, uint32_t count, uint64_t offset,
                            uint32_t flags, int *err)
-  __attribute__ ((__nonnull__ (1, 2, 6)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 6));
 extern int backend_flush (struct context *c,
                           uint32_t flags, int *err)
-  __attribute__ ((__nonnull__ (1, 3)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3));
 extern int backend_trim (struct context *c,
                          uint32_t count, uint64_t offset, uint32_t flags,
                          int *err)
-  __attribute__ ((__nonnull__ (1, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 5));
 extern int backend_zero (struct context *c,
                          uint32_t count, uint64_t offset, uint32_t flags,
                          int *err)
-  __attribute__ ((__nonnull__ (1, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 5));
 extern int backend_extents (struct context *c,
                             uint32_t count, uint64_t offset, uint32_t flags,
                             struct nbdkit_extents *extents, int *err)
-  __attribute__ ((__nonnull__ (1, 5, 6)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 5, 6));
 extern int backend_cache (struct context *c,
                           uint32_t count, uint64_t offset,
                           uint32_t flags, int *err)
-  __attribute__ ((__nonnull__ (1, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 5));
 
 /* plugins.c */
 typedef struct nbdkit_plugin *(*plugin_init_function) (void);
 extern struct backend *plugin_register (size_t index, const char *filename,
                                         void *dl,
                                         plugin_init_function plugin_init)
-  __attribute__ ((__nonnull__ (2, 3, 4)));
+  NBDKIT_ATTRIBUTE_NONNULL ((2, 3, 4));
 
 /* filters.c */
 typedef struct nbdkit_filter *(*filter_init_function) (void);
 extern struct backend *filter_register (struct backend *next, size_t index,
                                         const char *filename, void *dl,
                                         filter_init_function filter_init)
-  __attribute__ ((__nonnull__ (1, 3, 4, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3, 4, 5));
 
 /* locks.c */
 extern unsigned thread_model;
@@ -614,17 +614,17 @@ extern void unlock_unload (void);
 
 /* sockets.c */
 DEFINE_VECTOR_TYPE (sockets, int);
-extern void bind_unix_socket (sockets *) __attribute__ ((__nonnull__ (1)));
-extern void bind_tcpip_socket (sockets *) __attribute__ ((__nonnull__ (1)));
-extern void bind_vsock (sockets *) __attribute__ ((__nonnull__ (1)));
+extern void bind_unix_socket (sockets *) NBDKIT_ATTRIBUTE_NONNULL ((1));
+extern void bind_tcpip_socket (sockets *) NBDKIT_ATTRIBUTE_NONNULL ((1));
+extern void bind_vsock (sockets *) NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern void accept_incoming_connections (const sockets *socks)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 /* threadlocal.c */
 extern void threadlocal_init (void);
 extern void threadlocal_new_server_thread (void);
 extern void threadlocal_set_name (const char *name)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 extern const char *threadlocal_get_name (void);
 extern void threadlocal_set_instance_num (size_t instance_num);
 extern size_t threadlocal_get_instance_num (void);
