@@ -59,6 +59,7 @@ enum {
   TIMEOUT_OPTION,
   TLS_OPTION,
   TLS_CERTIFICATES_OPTION,
+  TLS_PRIORITY_OPTION,
   TLS_PSK_OPTION,
   TLS_VERIFY_PEER_OPTION,
   VSOCK_OPTION,
@@ -116,6 +117,7 @@ static const struct option long_options[] = {
   { "time-out",         required_argument, NULL, TIMEOUT_OPTION },
   { "tls",              required_argument, NULL, TLS_OPTION },
   { "tls-certificates", required_argument, NULL, TLS_CERTIFICATES_OPTION },
+  { "tls-priority",     required_argument, NULL, TLS_PRIORITY_OPTION },
   { "tls-psk",          required_argument, NULL, TLS_PSK_OPTION },
   { "tls-verify-peer",  no_argument,       NULL, TLS_VERIFY_PEER_OPTION },
   { "unix",             required_argument, NULL, 'U' },
