@@ -38,6 +38,8 @@
 
 source ./functions.sh
 set -e
+set -x
+set -u
 
 requires $CUT --version
 
