@@ -31,7 +31,9 @@
 # SUCH DAMAGE.
 
 source ./functions.sh
+#set -e
 set -x
+set -u
 
 requires_filter exitwhen
 requires nbdsh --version
